#!/etc/opt/openit/perl


print "-- Initiating update to licenseevents-individual-usage.conf...\n";

my @changevalue_def = (
	"\t\tChangeValue:serialnumber()=\"\"\n",
	"\t\t{\n",
	"\t\t\tfilter(String)=\"ChangeValue\"{}\n",
	"\t\t\tlimiting(String)=\"serial_number > 0\"{}\n",
	"\t\t\tset-value(String)=\"feature_version=\$feature_version . '_' . serial_number\"{}\n",
	"\t\t\tfields(String)=\"\"{}\n",
	"\t\t}\n"
);

my $filter = "/var/opt/openit/etc/filtrator/licenseevents-individual-usage.conf";

if( ! -e $filter ) {
	print "-- Failed to detect '$filter'; Skipping to next filter file...\n";
	exit;
}

my @filter_lines = read_file( $filter );

# Check if filter file has already been updated
if( grep { /ChangeValue:serialnumber/ } @filter_lines ) {
	print "-- '$filter' has already been updated prior. Moving to next filter file...\n";
	exit;
}

my $index = 0;

# Traverse to location after the filter-process line
$index = locate_line( $index, "filter-process", \@filter_lines );
$index++;

# Insert new filter process
@filter_lines[$index] =~ s/{\n/{\n\t\tChangeValue:serialnumber()=""\n\t\t{\n/;

# Insert closing curly brackets for new filter process
$index = locate_line( $index, "^\t}\n", \@filter_lines );
@filter_lines[$index] =~ s/}\n/}\n\t}\n/;

# Traverse to location after filter-defs line
$index = locate_line( $index, "filter-defs", \@filter_lines );
$index += 2;

# Insert filter definition for ChangeValue:serialnumber
splice @filter_lines, $index, 0, @changevalue_def;

# Insert code for adding "serial_number" in compare nodes here
@filter_lines = insert_in_compare( "CalcSum.product", \@filter_lines );
@filter_lines = insert_in_compare( "CalcSum.feature", \@filter_lines );
@filter_lines = insert_in_compare( "RemoveClass.feature", \@filter_lines );
@filter_lines = insert_in_compare( "Concurrent.feature", \@filter_lines );
@filter_lines = insert_in_compare( "Summarize.license-usage", \@filter_lines );

save_file( $filter, 2187, @filter_lines );
print "-- Done updating '$filter'\n";




# Subroutines -----------


sub insert_in_compare {
	my ( $target_filter, $lines_ref ) = @_;
	
	my $index = 0;
	$index = locate_line( $index, "filter-defs", \@$lines_ref );
	$index = locate_line( $index, $target_filter, \@$lines_ref );
	$index = locate_line( $index, "compare.String", \@$lines_ref );

	# Match 2nd double quote in compare(String) line
	my $n = 2;
	@$lines_ref[$index] =~ s/(")/!--$n ? ',serial_number"' : $1/ge;

	$target_filter =~ s/\./\:/;
	print "- Added serial_number in $target_filter\n";

	return @$lines_ref;
}

sub read_file {
	my ( $cfg ) = @_;
	open( IN, "$cfg" ) || die "ERROR: Unable to read cfg '$cfg'!\n";
	my @lines = <IN>;
	close( IN );
	return @lines;
}

sub locate_line {
	# For carets, colons and parentheses, use the . character to represent them
	# For the 3rd argument, always add a \ as prefix to the array
	my ( $start, $find, $lines_ref ) = (@_);
	my ( $index ) = grep { @$lines_ref[$_] =~ /$find/ } $start ..@$lines_ref;
	return $index;
}

sub save_file {
	my ( $cfg, $patch, @lines ) = @_;
	my $old = "$cfg.pre_$patch";
	my $new = "$cfg.new";
	
	open( OUT, ">$new" ) || die "ERROR: Can't write new cfg '$new'!\n";
	print OUT @lines;
	close( OUT );
	
	if( ! rename( $cfg, $old ) ) {
		die "ERROR: Unable to rename previous config '$cfg' to '$old'!\n";
	}
	
	if( ! rename( $new, $cfg ) ) {
		die "ERROR: Unable to rename new config to '$cfg'!\n";
	}
}
